/**
 * @filename:PurchaseOrderServiceImpl 2024-09-10
 * @project KJT v1.0.0 Copyright(c) 2024 wanglang Co. Ltd. All right reserved.
 */
package com.trade.service.order.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.trade.common.utils.ConstantUtil;
import com.trade.common.utils.StringUtils;
import com.trade.common.utils.bean.BeanCopyUtils;
import com.trade.common.utils.bean.Builder;
import com.trade.service.FileTypeEnum;
import com.trade.service.filestorage.entity.BusinessRelateFile;
import com.trade.service.filestorage.service.BusinessRelateFileBaseService;
import com.trade.service.order.domain.PurchaseOrder;
import com.trade.service.order.domain.PurchaseOrderCommodity;
import com.trade.service.order.domain.PurchaseOrderLogistics;
import com.trade.service.order.dto.PurchaseOrderDto;
import com.trade.service.order.dto.PurchaseOrderLogisticsDto;
import com.trade.service.order.dto.PurchaseSearchDto;
import com.trade.service.order.mapper.PurchaseOrderMapper;
import com.trade.service.order.service.PurchaseOrderCommodityService;
import com.trade.service.order.service.PurchaseOrderLogisticsService;
import com.trade.service.order.service.PurchaseOrderService;
import com.trade.service.order.vo.PurchaseAcountVo;
import com.trade.service.order.vo.PurchaseOrderDetailsVo;
import com.trade.service.order.vo.PurchaseOrderLogisticsVo;
import com.trade.service.order.vo.PurchaseOrderVo;
import com.trade.service.purchase.domain.PurchaseCommodity;
import com.trade.service.purchase.domain.PurchaseContract;
import com.trade.service.purchase.service.PurchaseCommodityBaseService;
import com.trade.service.purchase.service.PurchaseContractBaseService;
import com.trade.service.sales.domain.ContractFile;
import com.trade.service.sales.domain.ContractTerms;
import com.trade.service.sales.domain.SalesCommodity;
import com.trade.service.sales.domain.SalesContract;
import com.trade.service.sales.service.ContractTermsService;
import com.trade.service.sales.service.SalesContractService;
import com.trade.service.salesorder.domain.SalesOrder;
import com.trade.service.salesorder.service.ISalesOrderService;
import com.trade.service.serialNum.service.SerialNumberGenerator;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
 * @Description: 订单信息服务层实现类
 * @version: v1.0.0
 * @author: wanglang
 */
@Service
public class PurchaseOrderServiceImpl extends ServiceImpl<PurchaseOrderMapper, PurchaseOrder> implements PurchaseOrderService {

    @Resource
    private PurchaseOrderLogisticsService purchaseOrderLogisticsService;
    @Resource
    private ISalesOrderService salesOrderService;
    @Resource
    private SalesContractService salesContractService;
    @Resource
    private PurchaseContractBaseService purchaseContractBaseService;
    @Resource
    private PurchaseOrderCommodityService purchaseOrderCommodityService;
    @Resource
    private BusinessRelateFileBaseService businessRelateFileBaseService;
    @Resource
    private SerialNumberGenerator serialNumberGenerator;
    @Resource
    private ContractTermsService contractTermsService;
    @Resource
    private PurchaseCommodityBaseService purchaseCommodityBaseService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveOrderLogistics(PurchaseOrderLogisticsDto purchaseOrderLogisticsDto) {
        PurchaseOrderLogistics purchaseOrderLogistics = BeanCopyUtils.copy(purchaseOrderLogisticsDto,
                PurchaseOrderLogistics.class);
        BusinessRelateFile businessRelateFile = Builder.of(BusinessRelateFile.class)
                .with(BusinessRelateFile::setBelongId, purchaseOrderLogistics.getPurchaseOrderId())
                .with(BusinessRelateFile::setBelongType, ConstantUtil.PURCHASE_ORDER)
                .with(BusinessRelateFile::setFileType, FileTypeEnum.PURCHASE_ORDER_TRANS_PROVE.getCode())
                .with(BusinessRelateFile::setFileId, purchaseOrderLogisticsDto.getTransportCertificateFileId()).build();
        purchaseOrderLogisticsService.save(purchaseOrderLogistics);
        businessRelateFileBaseService.upsertBusinessRelateFiles(businessRelateFile);
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveOrder(List<PurchaseOrderDto> purchaseOrderDtos) {
        for (PurchaseOrderDto purchaseOrderDto : purchaseOrderDtos) {
            PurchaseOrder purchaseOrder = new PurchaseOrder();
            purchaseOrder.setSalesOrderId(purchaseOrderDto.getSalesOrderId());
            purchaseOrder.setCustomerUserId(purchaseOrderDto.getCustomerUserId());
            purchaseOrder.setSalesOrderId(purchaseOrderDto.getSalesOrderId());
            purchaseOrder.setEntityNo(purchaseOrderDto.getVendorEntityNo());
            purchaseOrder.setDeleted(0);
            List<PurchaseOrderCommodity> commodities = purchaseOrderDto.getCommodities();
            BigDecimal amount = commodities.stream().map(PurchaseOrderCommodity::getAmount).reduce(BigDecimal::add)
                    .orElse(BigDecimal.ZERO);
            BigDecimal taxRefundRate = commodities.stream()
                    .map(s -> s.getAmount().multiply(s.getTaxRefundRate()).multiply(new BigDecimal(s.getNumber())))
                    .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
            purchaseOrder.setAmountTotal(amount);
            purchaseOrder.setTaxAmountTotal(taxRefundRate);
            SalesOrder salesOrder = salesOrderService.getById(purchaseOrderDto.getSalesOrderId());
            SalesContract salesContract = salesContractService.getById(salesOrder.getSaleContractId());
            purchaseOrder.setAgencyCertNo(salesContract.getAgencyCertNo());
            PurchaseContract purchaseContract;
            if (StringUtils.isNotEmpty(purchaseOrder.getEntityNo())) {
                purchaseContract = purchaseContractBaseService.lambdaQuery()
                        .eq(PurchaseContract::getSalesContractNo, salesContract.getContractNo())
                        .eq(PurchaseContract::getVendorEntityNo, purchaseOrder.getEntityNo())
                        .eq(PurchaseContract::getDeleted, 0).one();
            } else {
                purchaseContract = purchaseContractBaseService.lambdaQuery()
                        .eq(PurchaseContract::getSalesContractNo, salesContract.getContractNo())
                        .eq(PurchaseContract::getDeleted, 0).one();
            }
            purchaseOrder.setPurchaseContractNo(purchaseContract.getContractNo());
            ContractTerms contractTerms = contractTermsService.getByContractIdAndType(purchaseContract.getId(),
                    ConstantUtil.SALES_CONTRACT_NO_PREFIX);
            purchaseOrder.setInvoiceType(contractTerms.getInvoiceType());
            purchaseOrder.setSettlementCycle(contractTerms.getSettlementCycle());
            purchaseOrder.setOrderNo(
                    serialNumberGenerator.nextValue(ConstantUtil.PURCHASE_ORDER_NO_PREFIX, salesOrder.getOrderNo()));
            List<PurchaseCommodity> purchaseCommodities = purchaseCommodityBaseService.lambdaQuery()
                    .eq(PurchaseCommodity::getPurchaseContractId, purchaseContract.getId())
                    .eq(PurchaseCommodity::getSalesContractId, 0).eq(PurchaseCommodity::getDeleted, 0).list();
            for (PurchaseOrderCommodity commodity : commodities) {
                PurchaseCommodity purchaseCommodity = purchaseCommodities.stream()
                        .filter(p -> p.getSalesCommodityId().equals(commodity.getSalesCommodityId())).findAny()
                        .orElse(null);
                if (purchaseCommodity != null) {
                    commodity.setPriceUnit(purchaseCommodity.getPriceUnit());
                }
            }
            this.baseMapper.insert(purchaseOrder);
            purchaseOrderCommodityService.saveBatch(commodities);
        }
        return true;
    }

    @Override
    public List<PurchaseOrderVo> selectByCondition(PurchaseSearchDto purchaseSearch) {
        return this.baseMapper.selectByCondition(purchaseSearch);
    }

    @Override
    public PurchaseOrderDetailsVo getDetailsById(Long id) {
        PurchaseOrderDetailsVo purchaseOrderDetailsVo = this.baseMapper.selectDetailsById(id);
        List<PurchaseOrderCommodity> commodities = purchaseOrderCommodityService.lambdaQuery()
                .eq(PurchaseOrderCommodity::getPurchaseOrderId, id).list();
        purchaseOrderDetailsVo.setCommodityList(commodities);
        PurchaseOrderLogisticsVo purchaseOrderLogisticsVo = purchaseOrderLogisticsService.queryByPurchaseOrderId(id);
        purchaseOrderDetailsVo.setLogisticsVo(purchaseOrderLogisticsVo);
        BusinessRelateFile businessRelateFile = businessRelateFileBaseService.getByCondition(
                purchaseOrderLogisticsVo.getId(), ConstantUtil.PURCHASE_ORDER,
                FileTypeEnum.PURCHASE_ORDER_TRANS_PROVE.getCode());
        purchaseOrderLogisticsVo.setId(businessRelateFile.getId());
        PurchaseContract purchaseContract = purchaseContractBaseService.lambdaQuery()
                .eq(PurchaseContract::getContractNo, purchaseOrderDetailsVo.getPurchaseContractNo())
                .eq(PurchaseContract::getDeleted, 0).one();
        ContractTerms contractTerms = contractTermsService.getByContractIdAndType(purchaseContract.getId(),
                ConstantUtil.PURCHASE_CONTRACT_CODE);
        PurchaseAcountVo purchaseAcountVo = new PurchaseAcountVo();
        purchaseAcountVo.setAccountName(contractTerms.getAccountName());
        purchaseAcountVo.setAccountNumber(contractTerms.getAccountNumber());
        purchaseAcountVo.setOpeningBank(contractTerms.getOpeningBank());
        purchaseOrderDetailsVo.setPurchaseAcountVo(purchaseAcountVo);
        return purchaseOrderDetailsVo;
    }
}
